//
// Licensed to the Apache Software Foundation (ASF) under one or more
// contributor license agreements.  See the NOTICE file distributed with
// this work for additional information regarding copyright ownership.
// The ASF licenses this file to You under the Apache License, Version 2.0
// (the "License"); you may not use this file except in compliance with
// the License.  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

//  .Net StockTrader Sample WCF Application for Benchmarking, Performance Analysis and Design Considerations for Service-Oriented Applications

using System.ServiceModel;
using Trade.ConfigServiceDataContract;

namespace Trade.ConfigServiceContract
{
    /// <summary>
    /// This is the service contract for the Configuration Service. It defines the service layer operations
    /// that are separately implemented in the implementation class.
    /// </summary> 
    [ServiceContract(Name = "ConfigServiceService", Namespace = "http://Trade.TraderConfigHost")]
    public interface IConfigService
    {

        /// <summary>
        /// Get configurations of a client
        /// </summary>
        /// <param name="client">Name of the client</param>
        /// <returns>Object ClientConfigRequest</returns>
        [OperationContract]
        ClientConfigResponse GetClientConfig(ClientConfigRequest client);

        /// <summary>
        /// Get configurations of a business service
        /// </summary>
        [OperationContract]
        BSConfigResponse GetBSConfig(BSConfigRequest bs);

        /// <summary>
        /// Get configurations of an order processor service
        /// </summary>
        [OperationContract]
        OPSConfigResponse GetOPSConfig(OPSConfigRequest ops);

        /// <summary>
        /// Sets connection from client to business service
        /// </summary>
        [OperationContract(Action = "ClientToBS")]
        void SetClientToBS(ClientToBS clientConfig);

        /// <summary>
        /// Sets connection from business service to order processor service
        /// </summary>
        [OperationContract(Action = "BSToOPS")]
        void SetBSToOPS(BSToOPS bsConfig);

        /// <summary>
        /// Gets all business service locations
        /// </summary>
        [OperationContract]
        ServiceLocation[] GetBSLocations();

        /// <summary>
        /// Gets all the order processing service locations
        /// </summary>
        [OperationContract]
        ServiceLocation[] GetOPSLocations();

        /// <summary>
        /// Sets service location (End point URL) of a given service.
        /// </summsry>
        /// <param name="location">Service Location object consisting of a ServiceName, ServiceUrl and SEC value</param>
        [OperationContract(Action = "ServiceLocation")]
        void SetServiceLocation(ServiceLocation location);

        /*************************************************************************************
         * TODO: The rest of these methods have been defined in the PHP Configuration Class.
         * They may need to be defined for the user interface

        /// <summary>
        /// Set the database configuration of a DB
        /// </summary>
        /// <param name="config">DBConfig object consisting of the DBName, DBPort and DBHostname that is to be set</param>
        [OperationContract(Action = "DBConfig")]
        void SetDbConfig(DBConfig config);

        /// <summary>
        /// Look up all the databases set up
        /// </summary>
        /// <returns>DBConfigs for all the DBs</returns>
        [OperationContract]
        DBConfig[] GetDBConfigs();

        /// <summary> 
        /// Sets active database.
        /// </summary>
        /// <param name="activeConfig">Name of the active database</param>
        [OperationContract(Action = "ActiveDB")]
        void SetActiveDB(string activeConfig);

        /// <summary>
        /// Get active database.
        /// </summary>
        [OperationContract]
        DBConfig GetActiveDB();

        /// <summary>
        /// Sets service location (End point URL) of a given service.
        /// </summsry>
        /// <param name="location">Service Location object consisting of a ServiceName, ServiceUrl and SEC value</param>
        [OperationContract(Action = "ServiceLocation")]
        void SetServiceLocation(ServiceLocation location);
         
        /// <summary>
        /// Get all connections from client to business service
        /// </summary>
        [OperationContract(Action = "GetClientToBSConnections")]
        ClientToBS[] GetClientToBsConnections();


        /// <summary>
        /// Get all connections from business service to order processor service
        /// </summary>
        [OperationContract(Action = "GetBSToOPSConnections")]
        BSToOPS[] GetBSToOPSConnections();

        ********************************************************************************/

    }

    

    
}

