//
// Licensed to the Apache Software Foundation (ASF) under one or more
// contributor license agreements.  See the NOTICE file distributed with
// this work for additional information regarding copyright ownership.
// The ASF licenses this file to You under the Apache License, Version 2.0
// (the "License"); you may not use this file except in compliance with
// the License.  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

//  .Net StockTrader Sample WCF Application for Benchmarking, Performance Analysis and Design Considerations for Service-Oriented Applications

//======================================================================================================
// The OrderProcessor ServiceContract, which is quite simple.
//======================================================================================================
//======================================================================================================
// Code originally contributed by Microsoft Corporation.
// This contribution to the Stonehenge project is limited strictly 
// to the source code that is submitted in this submission.  
// Any technology, including underlying platform technology, 
// that is referenced or required by the submitted source code 
// is not a part of the contribution.  
// For example and not by way of limitation, 
// any systems/Windows libraries (WPF, WCF, ASP.NET etc.) 
// required to run the submitted source code is not a part of the contribution
//======================================================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;
using System.Threading;
using Trade.BusinessServiceDataContract;

namespace Trade.OrderProcessorContract
{
    /// <summary>
    /// This is the service contract for the Order Processor Service. It defines the service layer operations
    /// that are separately implemented in the implementation class.
    /// </summary> 
    [ServiceContract(Name = "OrderProcessorService", Namespace = "http://Trade.TraderOrderHost")]
    public interface IOrderProcessor
    {
        //IsOneWay Marks Methods as Asynchrounous.
        //This is our non-transacted method for async calls via http, tcp, and volatile (in-process) msmq.
        [OperationContract(Action = "SubmitOrder", IsOneWay = true)]
        void SubmitOrder(OrderDataModel order);

        /// <summary>
        /// Online check method.
        /// </summary>
        [OperationContract(Action = "isOnline", IsOneWay = true)]
        void isOnline();
    }
}

