/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 package org.apache.stonehenge.stocktrader;

import com.ibm.websphere.samples.trade.*;

import java.util.List;
import java.util.ArrayList;

public class BeanFactory {
    public AccountDataBean toAccount(CustomAccountBean bean) {
        AccountDataBean newBean = new AccountDataBean();
        newBean.setAccountID(bean.getAccountID());
        newBean.setProfileID(bean.getProfileID());
        newBean.setBalance(bean.getBalance());
        newBean.setCreationDate(bean.getCreationDate());
        newBean.setLastLogin(bean.getLastLogin());
        newBean.setLoginCount(bean.getLoginCount());
        newBean.setLogoutCount(bean.getLogoutCount());
        newBean.setOpenBalance(bean.getOpenBalance());

        return newBean;
    }

    public AccountProfileDataBean toAccountProfile(CustomAccountProfileBean bean) {
        AccountProfileDataBean newBean = new AccountProfileDataBean();
        newBean.setAddress(bean.getAddress());
        newBean.setCreditCard(bean.getCreditCard());
        newBean.setEmail(bean.getEmail());
        newBean.setFullName(bean.getFullName());
        newBean.setPassword(bean.getPassword());
        newBean.setUserID(bean.getUserID());

        return newBean;
    }

    public CustomAccountProfileBean toCustomAccountProfileBean(AccountProfileDataBean bean) {
        CustomAccountProfileBean newBean = new CustomAccountProfileBean();
        newBean.setAddress(bean.getAddress());
        newBean.setCreditCard(bean.getCreditCard());
        newBean.setEmail(bean.getEmail());
        newBean.setFullName(bean.getFullName());
        newBean.setPassword(bean.getPassword());
        newBean.setUserID(bean.getUserID());

        return newBean;
    }

    public HoldingDataBean toHoldingData(CustomHoldingBean bean) {
        HoldingDataBean newBean = new HoldingDataBean();
        newBean.setHoldingID(bean.getHoldingID());
        newBean.setQuantity(bean.getQuantity());
        newBean.setPurchasePrice(bean.getPurchasePrice());
        newBean.setPurchaseDate(bean.getPurchaseDate());
        newBean.setQuoteID(bean.getQuoteID());

        return newBean;
    }

    public List<HoldingDataBean> toHoldingDataList(List<CustomHoldingBean> beans) {
        List<HoldingDataBean> newBeans = new ArrayList<HoldingDataBean>();
        for (CustomHoldingBean bean : beans) {
            newBeans.add(toHoldingData(bean));
        }
        
        return newBeans;
    }

    public MarketSummaryDataBeanWS toMarketSummaryData(CustomMarketSummaryBean bean) {
        MarketSummaryDataBeanWS newBean = new MarketSummaryDataBeanWS();

        newBean.setTSIA(bean.getOpenTSIA());
		newBean.setOpenTSIA(bean.getOpenTSIA());
		newBean.setVolume(bean.getVolume());
		newBean.setTopGainers(toArrayOfQuote(bean.getTopGainers()));
		newBean.setTopLosers(toArrayOfQuote(bean.getTopLosers()));

        return newBean;
    }

    private ArrayOfQuoteDataBean toArrayOfQuote(List<CustomQuoteBean> listOfCustomQuoteBean) {
        ArrayOfQuoteDataBean arrayOfQuoteDataBean = new ArrayOfQuoteDataBean();

        for (CustomQuoteBean customQuoteBean : listOfCustomQuoteBean) {
            arrayOfQuoteDataBean.getQuoteDataBean().add(toQuote(customQuoteBean));
        }

        return arrayOfQuoteDataBean;
    }

    public OrderDataBean toOrder(CustomOrderBean bean) {
        OrderDataBean newBean = new OrderDataBean();
        newBean.setCompletionDate(bean.getCompletionDate());
		newBean.setOpenDate(bean.getOpenDate());
		newBean.setOrderFee(bean.getOrderFee());
		newBean.setOrderID(bean.getOrderID());
		newBean.setOrderStatus(bean.getOrderStatus());
		newBean.setOrderType(bean.getOrderType());
		newBean.setPrice(bean.getPrice());
		newBean.setQuantity(bean.getQuantity());
		newBean.setSymbol(bean.getSymbol());

        return newBean;
    }

    public List<OrderDataBean> toOrderList(List<CustomOrderBean> beans) {
        List<OrderDataBean> newBeans = new ArrayList<OrderDataBean>();
        for (CustomOrderBean bean : beans) {
            newBeans.add(toOrder(bean));
        }

        return newBeans;
    }

    public QuoteDataBean toQuote(CustomQuoteBean bean) {
        QuoteDataBean newBean = new QuoteDataBean();

        newBean.setSymbol(bean.getSymbol());
		newBean.setCompanyName(bean.getCompanyName());
		newBean.setVolume(bean.getVolume());
		newBean.setPrice(bean.getPrice());
		newBean.setOpen(bean.getOpen());
		newBean.setLow(bean.getLow());
		newBean.setHigh(bean.getHigh());
		newBean.setChange(bean.getChange());
        
        return newBean;
    }
}