/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 package org.apache.stonehenge.stocktrader.services;

import org.apache.stonehenge.stocktrader.dal.ConfigServiceDAO;
import org.apache.stonehenge.stocktrader.dal.ConfigServiceDaoFactory;
import org.datacontract.schemas._2004._07.trade.BSConfigRequest;
import org.datacontract.schemas._2004._07.trade.BSConfigResponse;
import org.datacontract.schemas._2004._07.trade.ClientConfigRequest;
import org.datacontract.schemas._2004._07.trade.ClientConfigResponse;
import org.tempuri.ConfigService;
import org.tempuri.ConfigServiceService;
import traderconfighost.trade.GetBSConfig;
import traderconfighost.trade.GetBSConfigResponse;
import traderconfighost.trade.GetClientConfig;
import traderconfighost.trade.GetClientConfigResponse;

import javax.xml.ws.BindingProvider;

public class TradeConfigServiceClient {

    private static TradeConfigServiceClient self = null;
    private static ConfigServiceService service = null;
    private static ConfigServiceDaoFactory configServiceDaoFactory;
    private static ConfigServiceDAO configServiceDAO;

    public static TradeConfigServiceClient getInstance() {
            configServiceDaoFactory = ConfigServiceDaoFactory.getFacotry();
            configServiceDAO = configServiceDaoFactory.getConfigServiceDAO();
            self = new TradeConfigServiceClient();
        ((BindingProvider) service).getRequestContext()
                .put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, configServiceDAO.getConfigService());
        return self;
    }

    private TradeConfigServiceClient() {
        ConfigService configService = new ConfigService();
        service = configService.getBasicHttpBindingConfigServiceService();
    }

    public TradeOrderConfig getTradeOrderConfig() {
        GetClientConfig getClientConfig = new GetClientConfig();
        ClientConfigRequest clientconfigRequest = new ClientConfigRequest();
        clientconfigRequest.setClientName("METRO_CLIENT");
        getClientConfig.setClient(clientconfigRequest);
        GetClientConfigResponse clientConfigResponse = service.getClientConfig(getClientConfig);
        ClientConfigResponse clientresponse = clientConfigResponse.getGetClientConfigResult();
        String bsName = clientresponse.getBSName();

        GetBSConfig getBSConfig = new GetBSConfig();
        BSConfigRequest configRequest = new BSConfigRequest();
        configRequest.setBSName(bsName);
        getBSConfig.setBs(configRequest);
        GetBSConfigResponse getBSConfigResponse = service.getBSConfig(getBSConfig);
        BSConfigResponse response = getBSConfigResponse.getGetBSConfigResult();

        TradeOrderConfig config = new TradeOrderConfig();
        config.setServiceName(response.getOPSName());
        config.setEndpointURL(response.getOPS());
        config.setSecure(response.isSec());
        return config;
    }

//    public static void main(String arg[]) {
//        TradeConfigServiceClient client = TradeConfigServiceClient.getInstance();
//        TradeOrderConfig config = client.getTradeOrderConfig();
//        String url = config.getEndpointURL();
//
//        assert url.equals("http://localhost:8091/order_processor/OrderProcessorSec");
//
//        System.out.println("connect successfully!!");
//    }
}
