/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.stonehenge.stocktrader.dal;

import java.math.BigDecimal;

import org.apache.stonehenge.stocktrader.CustomHoldingBean;
import org.apache.stonehenge.stocktrader.CustomOrderBean;
import org.apache.stonehenge.stocktrader.CustomQuoteBean;

public interface OrderDAO {

	public CustomQuoteBean getQuoteForUpdate(String symbol) throws DAOException;

	public int createHolding(CustomOrderBean order) throws DAOException;

	public void updateHolding(int holdingId, double quantity)
			throws DAOException;

	public void deleteHolding(int holdingId) throws DAOException;

	public CustomHoldingBean getHoldingForUpdate(int orderId)
			throws DAOException;

	public CustomHoldingBean getHolding(int holdingId) throws DAOException;

	public void updateAccountBalance(int accountId, BigDecimal total)
			throws DAOException;

	public CustomOrderBean createOrder(String userID, String symbol,
			String orderType, double quantity, int holdingID)
			throws DAOException;

	public void updateOrder(CustomOrderBean order) throws DAOException;

	public void updateStockPriceVolume(double quantity, CustomQuoteBean quote)
			throws DAOException;

	public void closeOrder(CustomOrderBean order) throws DAOException;

    public void beginTransaction() throws DAOException;

    public void commitTransaction() throws DAOException;

    public void rollbackTransaction() throws DAOException;

    public void close() throws DAOException;
}
