<?php
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
require_once ("order_processor.php");

// define the class map
$class_map = array("anyType" => "anyType", "OrderDataBean" => "OrderDataBean", 
	"SubmitOrder" => "SubmitOrder", "isOnline" => "isOnline");

class isOnline
{
}


/**
 * This method corresponds to the primary operation of the service.It processes
 * an incomming order object.
 * @param input, the order object, filled with data correspond to an order
 * @return accept upon success.
 */ 

function SubmitOrder($input) 
{
	SendAcceptNotification();
	if ($input->order != NULL)
	{
		if(($input->order->orderType == ORDER_TYPE_BUY) || 
			($input->order->orderType == ORDER_TYPE_SELL))
		{
			ProcessOrder($input->order);
		}
		else
		{
			error_log ("Incoming order type is incorrect.\n");
		}
	}
	else
	{
		error_log ("Incoming order request is NULL.\n");
	}
}

/**
 * This method obtains some additional time and let the business service
 * to store information regarding an order. Then the OrderProcessor can
 * continue processing the order request that was just stored in the 
 * database.
 */

function SendAcceptNotification()
{
	header('HTTP/1.1 202 Accepted');
	header('Content-Length: 0');
	flush();
	sleep(1);
}

/**
 * This function corresponds to the isOnline operation. This is used by the
 * Configuration service to make sure that the OrderProcessor service is 
 * online.
 * @return just a return is expected to make sure the service is online.
 */

function isOnline($input) 
{
	return;
}

// define the operations map
$operations = array("SubmitOrder" => "SubmitOrder", "isOnline" => "isOnline");

// define the actions => operations map
$actions = array("SubmitOrderOnePhase" => "SubmitOrder", 
	"isOnline" => "isOnline");

// create service in WSDL mode
$service = new WSService(array ("wsdl" =>"../resources/wsdl/TradeOrders.wsdl",
	"operations" => $operations,
    "actions" => $actions,
	"classmap" => $class_map,));

// process client requests and reply
$service->reply();
?>
